#ifndef INCtransferh
#define INCtransferh

#define MAX_STREAM 4

typedef struct block_header {              /* format of data block  */
        uint16_t hdr_flags;          /* see below */
        uint16_t hdr_stream;         /* =0 for forced ack request or 1=>MAX_STREAM  */
        uint16_t hdr_endian;         /*   data byte order */
        uint16_t hdr_spare;
        uint32_t  hdr_sequence;        /* for this stream */
        uint32_t  hdr_blocklength;     /*  total length of this block including the header */
        uint32_t  hdr_datalength;      /*  length of user data in the block  */
        uint32_t  hdr_offset;          /*  very large blocks may be fragmented  */
        uint32_t  hdr_id1;             /*  for spy to locate header  =0x19062002 */
        uint32_t  hdr_id2;             /*  for spy to locate header  =0x09592400 */
} HEADER;

typedef struct mbs_block_header {
        uint32_t  mbs_endian;
        uint32_t  mbs_blocksize;
} MBS_HEADER;


/*    I2O definitions for use at Legnaro   */
#ifdef MC68020
  #define BIG_ENDIAN__ true
#endif
#ifdef LYNXOS
  #define BIG_ENDIAN__ true
#endif
#ifdef LINUX
  #define LITTLE_ENDIAN__ true
#endif
#ifdef SOLARIS
  #define BIG_ENDIAN__ true
#endif
#if (defined WIN32 || defined GWIN32)
  #define LITTLE_ENDIAN__ true
#endif


typedef unsigned char U8;
typedef unsigned short U16;
typedef unsigned int U32;
typedef U32 BF;    /* bit fields */

typedef struct _U64 {
    U32   LowPart;
    U32   HighPart;
} U64;

#define MERGE_TID 15     /* application */
#define VXI_TID 14       /* source */
#define I2O_PRIVATE_MESSAGE     0xff
#define I2O_BU_GET  0x0017
#define XDAQ_ORGANIZATION_ID 0x100

typedef struct _I2O_SEND_TO_BU_MESSAGE_FRAME {
#if defined(LITTLE_ENDIAN__)
    U8       VersionOffset;
    U8       MsgFlags;
    U16      MessageSize;
    BF       TargetAddress:12;
    BF       InitiatorAddress:12;
    BF       Function:8;
    U64      InitiatorContext;
#elif defined(BIG_ENDIAN__)
    U16      MessageSize;
    U8       MsgFlags;
    U8       VersionOffset;
    BF       Function:8;
    BF       InitiatorAddress:12;
    BF       TargetAddress:12;
    U64      InitiatorContext;
#endif
    U64      TransactionContext;
#if defined(LITTLE_ENDIAN__)
    U16      XFunctionCode;
    U16      OrganizationID;
#elif defined(BIG_ENDIAN__)
    U16      OrganizationID;
    U16      XFunctionCode;
#endif
    U32      buff_seqn;
}  I2O_SEND_TO_BU_MESSAGE_FRAME, *PI2O_SEND_TO_BU_MESSAGE_FRAME;



/*
    flags   (bit 0 is lsb)
    bit 0       request = 0;  ack = 1
    bit 1       = 1  suppress ack;  =0 normal ack
    bit 2       = 1  force ack   (no data follows)
*/

#define HDR_ID1 0x19062002
#define HDR_ID2 0x09592400

typedef struct ack {
    unsigned short acq_flags;          /* bit 0 = 1 for ack  */
        unsigned short acq_code;           /* =0 for good ack otherwise error reason */
        unsigned short acq_ts_state;       /* tape server state - see below */
        unsigned short acq_stream;         /* =0 for forced ack ack or 1=>MAX_STREAM  */
        unsigned short acq_endian;         /* =1 transmitted in host byte order */
        unsigned short acq_spare;
        unsigned long acq_sequence;        /* for stream != 0 is sequence being acked */
        unsigned short acq_stream_state1;           /* see below  */
        unsigned short acq_stream_state2;           /*   */
        unsigned short acq_stream_state3;           /*   */
        unsigned short acq_stream_state4;           /*   */
        unsigned short acq_stream_window1;          /*   */
        unsigned short acq_stream_window2;          /*   */
        unsigned short acq_stream_window3;          /*   */
        unsigned short acq_stream_window4;          /*   */
} ACK;

        
int transferTxData (char *, unsigned long, int);
int transferTxRestart ();
int transferBlockSize(int);
int transferMode(int);
int transferEndian(int);
int transferPort(int);
int transferInit (char *);
void transferClose ();
int transferStatus ();
int transferSetVerbose(int);
int transferSetUser(int);
int transferNice(int);
int transferUseOverlap(int);
int transferTxWait();
int transferState();
int transferSndBufSize(int);

#endif
